@@ -10,6 +10,7 @@ import requests
 from kerasltisubmission.exceptions import (
     KerasLTISubmissionBadResponseException,
     KerasLTISubmissionConnectionFailedException,
+    KerasLTISubmissionException,
     KerasLTISubmissionInputException,
     KerasLTISubmissionInvalidSubmissionException,
     KerasLTISubmissionNoInputException,
@@ -60,7 +61,9 @@ class LTIProvider:
                 message=rr.get("error"),
             )
 
-    def guess(self, assignment_id: AnyIDType, predictions: PredictionsType) -> None:
+    def guess(
+        self, assignment_id: AnyIDType, predictions: PredictionsType
+    ) -> typing.Tuple[float, float]:
         log.debug(
             f"Submitting {len(predictions)} predictions to the provider for grading"
         )
@@ -85,12 +88,17 @@ class LTIProvider:
             raise KerasLTISubmissionConnectionFailedException(
                 self.submission_api_endpoint, e
             ) from None
-        if r.status_code == 200 and rr.get("error") is None:
+        try:
+            assert r.status_code == 200 and rr.get("error") is None
             log.debug(
                 f"Sent {len(predictions)} predictions to the provider for grading"
             )
             log.info(f"Successfully submitted assignment {assignment_id} for grading")
-        else:
+            return (
+                round(rr.get("accuracy"), ndigits=2),
+                round(rr.get("grade"), ndigits=2),
+            )
+        except (AssertionError, KeyError, ValueError, TypeError):
             raise KerasLTISubmissionBadResponseException(
                 api_endpoint=self.submission_api_endpoint,
                 return_code=r.status_code,
@@ -98,14 +106,26 @@ class LTIProvider:
                 message=rr.get("error"),
             )
 
+    @staticmethod
+    def safe_shape(
+        shape: typing.Tuple[typing.Optional[typing.Any], ...]
+    ) -> typing.Tuple[int, ...]:
+        escaped = []
+        for dim in shape:
+            escaped.append(-1 if not dim else dim)
+        return tuple(escaped)
+
     def submit(
         self,
         s: typing.Union["Submission", typing.List["Submission"]],
         verbose: bool = True,
-        expected_input_shape: typing.Optional[
+        reshape: bool = True,
+        strict: bool = False,
+        expected_output_shape: typing.Optional[
             typing.Tuple[typing.Optional[typing.Any], ...]
         ] = None,
-    ) -> None:
+    ) -> typing.Dict[str, typing.Dict[str, float]]:
+        results = dict()
         if isinstance(s, list):
             submissions = s
         else:
@@ -113,11 +133,12 @@ class LTIProvider:
         for sub in submissions:
 
             if (
-                expected_input_shape
-                and not sub.model.output_shape == expected_input_shape
+                strict
+                and expected_output_shape
+                and not sub.model.output_shape == expected_output_shape
             ):
                 raise KerasLTISubmissionInputException(
-                    f"Model has invalid output shape: Got {sub.model.output_shape} but expected {expected_input_shape}"
+                    f"Model has invalid output shape: Got {sub.model.output_shape} but expected {expected_output_shape}"
                 )
 
             # Get assignment inputs and propagate errors
@@ -129,30 +150,45 @@ class LTIProvider:
                     self.input_api_endpoint, sub.assignment_id
                 )
 
-            input_shape = np.asarray([i.get("matrix") for i in inputs]).shape
+            input_matrices = np.asarray([i.get("matrix") for i in inputs])
+            input_shape = input_matrices.shape
             expected_input_shape = (None, *input_shape[1:])
             if sub.model.input_shape != expected_input_shape:
-                raise KerasLTISubmissionInputException(
-                    f"Input shape mismatch: Got {sub.model.input_shape} but expected {expected_input_shape}"
+                output_shape_mismatch = f"Input shape mismatch: Got {sub.model.input_shape} but expected {expected_input_shape}"
+                if not reshape:
+                    raise KerasLTISubmissionInputException(output_shape_mismatch)
+                # Try to reshape
+                log.warning(output_shape_mismatch)
+                input_matrices = input_matrices.reshape(
+                    self.safe_shape(sub.model.input_shape)
                 )
 
             predictions: PredictionsType = dict()
             if not verbose:
-                net_out = sub.model.predict(
-                    np.asarray([i.get("matrix") for i in inputs])
-                )
+                net_out = sub.model.predict(input_matrices)
                 predictions = {
                     str(inputs[i].get("hash")): int(np.argmax(net_out[i]))
                     for i in range(len(inputs))
                 }
             else:
-                for i in progressbar.progressbar(inputs, redirect_stdout=True):
-                    input_matrix = i.get("matrix")
-                    input_hash = i.get("hash")
-                    probabilities = sub.model.predict(
-                        np.expand_dims(np.asarray(input_matrix), axis=0)
-                    )
-                    prediction = np.argmax(probabilities)
-                    predictions[input_hash] = int(prediction)
+                errors: typing.List[Exception] = []
+                for i in progressbar.progressbar(
+                    range(len(inputs)), redirect_stdout=True
+                ):
+                    try:
+                        input_matrix = input_matrices[i]
+                        input_hash = inputs[i].get("hash")
+                        probabilities = sub.model.predict(
+                            np.expand_dims(np.asarray(input_matrix), axis=0)
+                        )
+                        prediction = np.argmax(probabilities)
+                        predictions[input_hash] = int(prediction)
+                    except Exception as e:
+                        if e not in errors:
+                            errors.append(e)
+                if len(errors) > 0:
+                    raise KerasLTISubmissionException()
 
-            self.guess(sub.assignment_id, predictions)
+            accuracy, grade = self.guess(sub.assignment_id, predictions)
+            results[str(sub.assignment_id)] = dict(accuracy=accuracy, grade=grade)
+        return results
